import { Http, Loader } from '@zeedhi/core';
import { InstanceService } from '@/services/InstanceService';
import { DialogServiceCustom } from '@/services/DialogServiceCustom';
import { LoadingService } from '@zeedhi/common';
import { AlertServiceCustom } from '@/services/AlertServiceCustom';

export class DMTabSolicitationEditController {
	public demandManagementController = Loader.getInstance('DemandManagementController');

	public defaultTimeoutTimer: any;

	public isCallTypeS() {
		return this.demandManagementController.callType === 'S';
	}

	public getFormEdit() {
		return InstanceService.getInstanceZdForm('formDemandManagementS');
	}

	/** Eventos de fields */

	public onChangeClientEditCallModal(params: any) { // CLIENT INPUT EDIT
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const productInstance = InstanceService.getInstanceZdSelect('productEdit');
			productInstance.value = '';
			if (params.value) {
				this.resetInputtedValuesByClientEditField();
				productInstance.value = '';
				productInstance.datasource.filter.CDCLIENTE = params.component.value;
				try {
					await productInstance.datasource.get();
					this.demandManagementController.clientNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.demandManagementController.clientNOTSelected = true;
					this.demandManagementController.productNOTSelected = true;
					this.demandManagementController.screenNameNOTSelected = true;
					this.demandManagementController.productTypeNOTSelected = true;
					this.demandManagementController.versionNOTSelected = true;
					this.demandManagementController.releaseNOTSelected = true;
				}
			} else {
				this.demandManagementController.clientNOTSelected = true;
				this.demandManagementController.productNOTSelected = true;
				this.demandManagementController.screenNameNOTSelected = true;
				this.demandManagementController.productTypeNOTSelected = true;
				this.demandManagementController.versionNOTSelected = true;
				this.demandManagementController.releaseNOTSelected = true;
				this.resetInputtedValuesByClientEditField();
			}
		}, 350);
	}

	public onChangeProductEditCallModal(params: any) {
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			if (params.value) {
				this.resetInputtedValuesByProductEditField();
				const customerInstance = InstanceService.getInstanceZdSelect('customerEdit');
				const categoryInstance = InstanceService.getInstanceZdSelect('categoryEdit');
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
				const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseEdit');
				categoryInstance.datasource.filter.APPLICATION_TYPE = 'S';

				productTypeInstance.datasource.filter.PRODUCT_CODE = params.component.selectedData.SGSISTEMA;
				useLicenseInstance.datasource.filter.CDCLIENTE = customerInstance.selectedData.CDCLIENTE;
				try {
					await categoryInstance.datasource.get();
					await productTypeInstance.datasource.get();
					await useLicenseInstance.datasource.get();
					this.demandManagementController.productNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.demandManagementController.productNOTSelected = true;
					this.demandManagementController.screenNameNOTSelected = true;
					this.demandManagementController.productTypeNOTSelected = true;
					this.demandManagementController.versionNOTSelected = true;
					this.demandManagementController.releaseNOTSelected = true;
				}
			} else {
				this.demandManagementController.productNOTSelected = true;
				this.demandManagementController.screenNameNOTSelected = true;
				this.demandManagementController.productTypeNOTSelected = true;
				this.demandManagementController.versionNOTSelected = true;
				this.demandManagementController.releaseNOTSelected = true;
				this.resetInputtedValuesByClientEditField();
			}
		}, 350);
	}

	public async onFocusProductEditCall() {
		const productInstance = InstanceService.getInstanceZdSelect('productEdit');
		delete productInstance.datasource.filter.CALL_RELATIONSHIP;
		delete productInstance.datasource.filter.CDPRODCOMP;

		await productInstance.datasource.get();
	}

	public onChangeProductTypeEditCallModal(params: any) { // PRODUCT TYPE EDIT
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
			const releaseInstance = InstanceService.getInstanceZdSelect('releaseEdit');
			const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
			if (params.value) {
				const productInstance = InstanceService.getInstanceZdSelect('productEdit');
				versionSInstance.datasource.filter.PRODUCT_TYPE = params.component.value;
				versionSInstance.datasource.filter.PRODUCT_CONCAT = `${params.component.selectedData.CDARVPROD}${productInstance.selectedData.SGSISTEMA}`;

				try {
					const version = await versionSInstance.datasource.get();
					if (version.length === 1) {
						this.demandManagementController.versionNOTSelected = false;
						await versionSInstance.setValue(version[0].ID, false);
						releaseInstance.datasource.filter.PRODUCT_TYPE = params.component.value;
						releaseInstance.datasource.filter.PRODUCT_VERSION = version[0].ID;
						const release = await releaseInstance.datasource.get();
						if (release.length === 1) {
							this.demandManagementController.releaseNOTSelected = false;
							releaseInstance.value = release[0].ID;
							patchInstance.datasource.filter.PRODUCT_TYPE = params.component.value;
							patchInstance.datasource.filter.PRODUCT_VERSION = version[0].ID;
							patchInstance.datasource.filter.PRODUCT_RELEASE = release[0].ID;
							await patchInstance.datasource.get();
						}
					}
					this.demandManagementController.productTypeNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.demandManagementController.productTypeNOTSelected = true;
					this.demandManagementController.versionNOTSelected = true;
					this.demandManagementController.releaseNOTSelected = true;
					versionSInstance.value = '';
					releaseInstance.value = '';
					patchInstance.value = '';
				}
			} else {
				this.demandManagementController.productTypeNOTSelected = true;
				this.demandManagementController.versionNOTSelected = true;
				this.demandManagementController.releaseNOTSelected = true;
				versionSInstance.value = '';
				releaseInstance.value = '';
				patchInstance.value = '';
			}
		}, 350);
	}

	public onChangeVersionSEditCallModal(params: any) {
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const releaseInstance = InstanceService.getInstanceZdSelect('releaseEdit');
			const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
			if (params.value) {
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
				releaseInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
				releaseInstance.datasource.filter.PRODUCT_VERSION = params.component.value;
				try {
					const release = await releaseInstance.datasource.get();
					if (release.length === 1) {
						releaseInstance.value = release[0].ID;
						this.demandManagementController.releaseNOTSelected = false;
						patchInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
						patchInstance.datasource.filter.PRODUCT_VERSION = params.component.value;
						patchInstance.datasource.filter.PRODUCT_RELEASE = release[0].ID;
						await patchInstance.datasource.get();
					}
					this.demandManagementController.versionNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.demandManagementController.versionNOTSelected = true;
					this.demandManagementController.releaseNOTSelected = true;
					releaseInstance.value = '';
					patchInstance.value = '';
				}
			} else {
				debugger;
				this.demandManagementController.versionNOTSelected = true;
				this.demandManagementController.releaseNOTSelected = true;
				releaseInstance.value = '';
				patchInstance.value = '';
			}
		}, 350);
	}

	public onChangeReleaseEditCallModal(params: any) {
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
			if (params.value) {
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
				const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');

				patchInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
				patchInstance.datasource.filter.PRODUCT_VERSION = versionSInstance.value;
				patchInstance.datasource.filter.PRODUCT_RELEASE = params.component.value;

				try {
					await patchInstance.datasource.get();
					this.demandManagementController.releaseNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.demandManagementController.releaseNOTSelected = true;
					patchInstance.value = '';
				}
			} else {
				this.demandManagementController.releaseNOTSelected = true;
				patchInstance.value = '';
			}
		}, 350);
	}

	public resetValidationsEDIT() {
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementS');
		callFormInstance.resetValidation();
	}

	public resetInputtedValuesByClientEditField() {
		const callFormInstance = this.getFormEdit();
		callFormInstance.value.categoryEdit = '';
		callFormInstance.value.productTypeEdit = '';
		callFormInstance.value.versionEditS = '';
		callFormInstance.value.releaseEdit = '';
		callFormInstance.value.patchEdit = '';
		callFormInstance.value.useLicenseEdit = '';
	}

	public resetInputtedValuesByProductEditField() {
		const callFormInstance = this.getFormEdit();
		callFormInstance.value.categoryEdit = '';
		callFormInstance.value.productTypeEdit = '';
		callFormInstance.value.versionEditS = '';
		callFormInstance.value.releaseEdit = '';
		callFormInstance.value.patchEdit = '';
	}

	public async saveEditCall() {
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementS');
		const checkValidation = callFormInstance.validate();
		if (checkValidation) {
			const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
			const customerInstance = InstanceService.getInstanceZdSelect('customerEdit');
			LoadingService.show();
			const productInstance = InstanceService.getInstanceZdSelect('productEdit');
			const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
			const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
			try {
				await Http.post('/productCallSave', {
					CALL_RELATIONSHIP: this.demandManagementController.callRelationship,
					ID: this.demandManagementController.callID,
					REFERENCE: callFormInstance.value.referenceEdit,
					PRODUCT_TYPE_NAME: productTypeInstance.selectedData.LABEL,
					MENU_ITEM: '',
					MENU_VERSION: '',
					NMRAZSOCCLIE: customerInstance.selectedData.NMRAZSOCCLIE,
					CALL_LABELS: callFormInstance.value.labelsEdit,
					DESCRIPTION: callFormInstance.value.descriptionEdit,
					REPORTED_BY: callFormInstance.value.reportedByEdit,
					CDOPERRESP: callFormInstance.value.responsibleByEdit,
					CDCLIENTE: customerInstance.selectedData.CDCLIENTE,
					PLATAFORMA: 'DELPHI',
					PROJECT_ID: callFormInstance.value.productEdit.PROJECT_ID,
					SGSISTEMA: productInstance.selectedData.SGSISTEMA,
					TYPE: 'S',
					PRODUCT: versionSInstance.selectedData.CDPRODCOMP,
					PRODUCT_VERSION: callFormInstance.value.versionEditS,
					PRODUCT_RELEASE: callFormInstance.value.releaseEdit,
					PRODUCT_TYPE: callFormInstance.value.productTypeEdit,
					CDARVPROD: productTypeInstance.selectedData.CDARVPROD,
					CLASSIFICATION: callFormInstance.value.categoryEdit,
					PRODUCT_PATCH: callFormInstance.value.patchEdit,
					USE_LICENSE: callFormInstance.value.useLicenseEdit,
					APPOINTMENT_DATE: callFormInstance.value.appointmentDateEdit,
				});
				AlertServiceCustom.defaultSaveSuccessAlert();
				await callGridInstance.datasource.get();
			} catch (e) {
				LoadingService.hide();
				DialogServiceCustom.showErrorBackend(e);
			}
			LoadingService.hide();
		} else {
			DialogServiceCustom.messageErrorValidate();
		}
	}
}
